/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.urlhandlers;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.Artifact;
import org.wso2.registry.RegistryException;
import org.wso2.registry.jdbc.ConnectionFactory;
import org.wso2.registry.jdbc.dao.CommentsDAO;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

/**
 * Handles URLs in the form <path>?comments.
 * e.g. /project/services/inventoryService?comments
 */
public class CommentCollectionURLHandler extends URLHandler {

    private static final Log log = LogFactory.getLog(CommentCollectionURLHandler.class);

    public CommentCollectionURLHandler(ConnectionFactory connectionFactory) {
        super(connectionFactory);
    }

    public boolean handleURL(String url, Artifact artifact) throws RegistryException {

        String[] parts = url.split("\\?");

        if (parts.length != 2) {
            return false;
        }

        if (parts[0].length() == 0) {
            return false;
        }

        if (!parts[1].equals("comments")) {
            return false;
        }

        if (parts[1].length() != "comments".length()) {
            return false;
        }

        String resourcePath = parts[0];

        Connection conn = connectionFactory.getConnection();
        CommentsDAO commentsDAO = new CommentsDAO();

        try {
            artifact.setPath(url);
            artifact.setDirectory(true);

            long[] commentIDs = commentsDAO.getCommentIDs(resourcePath, conn);
            List commentPaths = new ArrayList();
            for (int i = 0; i < commentIDs.length; i++) {
                String commentPath = resourcePath + "?comment" + commentIDs[i];
                commentPaths.add(commentPath);
            }
            String[] commentsContent =
                    (String[]) commentPaths.toArray(new String[commentPaths.size()]);
            artifact.setContent(commentsContent);

            return true;

        } catch (SQLException e) {
            String msg = "Failed to get comments for resource: " +
                    resourcePath + ". Caused by: " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
            
        } finally {
            try {
                conn.close();
            } catch (SQLException ignore) {

            }
        }
    }
}
